VERSION 5.00
Begin VB.UserControl FileNameBox 
   ClientHeight    =   285
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2415
   ScaleHeight     =   285
   ScaleWidth      =   2415
   ToolboxBitmap   =   "FileNameBox.ctx":0000
   Begin VB.CommandButton cmdBrowse 
      Height          =   285
      Left            =   1800
      Picture         =   "FileNameBox.ctx":00FA
      Style           =   1  'Graphical
      TabIndex        =   1
      ToolTipText     =   "Browse for file"
      Top             =   0
      Width           =   285
   End
   Begin VB.TextBox txtFilename 
      Height          =   285
      Left            =   0
      TabIndex        =   0
      Top             =   0
      Width           =   1815
   End
   Begin VB.CommandButton cmdFolder 
      Enabled         =   0   'False
      Height          =   285
      Left            =   1920
      Picture         =   "FileNameBox.ctx":0F3C
      Style           =   1  'Graphical
      TabIndex        =   2
      Top             =   0
      Visible         =   0   'False
      Width           =   285
   End
   Begin VB.CommandButton cmdFile 
      Enabled         =   0   'False
      Height          =   285
      Left            =   1920
      Picture         =   "FileNameBox.ctx":1D7E
      Style           =   1  'Graphical
      TabIndex        =   3
      Top             =   0
      Visible         =   0   'False
      Width           =   285
   End
End
Attribute VB_Name = "FileNameBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Input field for file or directory selection"
Option Explicit

Private Const COMPONENT As Long = 555
Private Const COMDLGMISSING As Long = 1024
Private Const BASEPATHMISMATCH As Long = 1025
Private Const EXTENSIONINVALID As Long = 1026
Private Const PATHDOESNOTEXIST As Long = 1027

Public Enum fnbType
  fnbFile
  fnbFolder
End Enum

Event Change()
Attribute Change.VB_Description = "This event is raised whenever the text changes."
Event MismatchedPath(ByRef Message As String, ByRef Title As String)
Attribute MismatchedPath.VB_Description = "Raised when a file is selected that is not inside BasePath.  If Message is set to """" then no message will be displayed."
Event InvalidExtension(ByRef Message As String, ByRef Title As String)
Attribute InvalidExtension.VB_Description = "Raised when a file with an extension other than DefaultExt is selected.  If Message is set to """" then no message will be displayed."

Private m_comdlg As CommonDialog
Private m_cdTitle As String
Private m_cdExt As String
Private m_cdFilter As String

Private m_select As Boolean
Private m_validate As Boolean
Private m_strip As Boolean
Private m_type As fnbType
Private m_base As String

Private Sub cmdBrowse_Click()
  Dim fn As String, t As String
  If m_comdlg Is Nothing Then
    Err.Raise vbObjectError + COMDLGMISSING, COMPONENT, LoadResString(COMDLGMISSING)
    Exit Sub
  End If
  m_comdlg.CancelError = True
  m_comdlg.DialogTitle = m_cdTitle
  If SelType = fnbFolder Then
    m_comdlg.FileName = DefaultExt
  Else
    m_comdlg.FileName = ""
  End If
  m_comdlg.InitDir = m_base
  m_comdlg.DefaultExt = m_cdExt
  m_comdlg.Filter = m_cdFilter
  On Error GoTo cancelledopen
  m_comdlg.ShowOpen
  On Error GoTo 0
  fn = m_comdlg.FileName
  If m_strip Then
    If StrComp(Left$(fn, Len(m_base)), m_base, vbTextCompare) = 0 Then
      fn = Mid$(fn, Len(m_base) + 1)
    ElseIf m_validate Then
      fn = LoadResString(BASEPATHMISMATCH)
      t = LoadResString(COMPONENT)
      RaiseEvent MismatchedPath(fn, t)
      If Len(fn) > 0 Then
        MsgBox fn, vbInformation, t
        txtFilename.Text = ""
        Exit Sub
      End If
    End If
    If m_type = fnbFile Then
      If StrComp(Right$(fn, Len(m_cdExt)), m_cdExt, vbTextCompare) = 0 Then
        fn = Left$(fn, Len(fn) - Len(m_cdExt))
      ElseIf m_validate Then
        fn = LoadResString(EXTENSIONINVALID)
        t = LoadResString(COMPONENT)
        RaiseEvent InvalidExtension(fn, t)
        If Len(fn) > 0 Then
          MsgBox fn, vbInformation, t
          txtFilename.Text = ""
          Exit Sub
        End If
      End If
    Else
      Dim c As Long
      c = Len(fn)
      While c > 0
        If Mid$(fn, c, 1) = "\" Then
          fn = Left$(fn, c)
          c = 0
        End If
        c = c - 1
      Wend
      If c = 0 Then
        fn = ""
      End If
    End If
  End If
  txtFilename.Text = fn
  RaiseEvent Change
  Exit Sub
cancelledopen:
  Exit Sub
End Sub

Private Sub txtFilename_Change()
  RaiseEvent Change
End Sub

Private Sub txtFilename_GotFocus()
  If AutoSelect Then
    With txtFilename
      .SelStart = 0
      .SelLength = Len(.Text)
    End With
  End If
End Sub

Private Sub UserControl_InitProperties()
  Text = Extender.Name
  Set Font = Ambient.Font
  ForeColor = Ambient.ForeColor
  AutoValidate = False
  StripBasePath = True
  SelType = fnbFile
  BasePath = ""
  DefaultExt = ""
  Filter = ""
  DialogTitle = ""
  Appearance = 1
  AutoSelect = False
  Enabled = True
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
  AutoValidate = PropBag.ReadProperty("AutoValidate", False)
  BasePath = PropBag.ReadProperty("BasePath", "")
  SelType = PropBag.ReadProperty("SelType", fnbFile)
  StripBasePath = PropBag.ReadProperty("StripBasePath", True)
  Text = PropBag.ReadProperty("Text", "")
  DefaultExt = PropBag.ReadProperty("DefaultExt", "")
  Filter = PropBag.ReadProperty("Filter", "")
  DialogTitle = PropBag.ReadProperty("DialogTitle", "")
  Appearance = PropBag.ReadProperty("Appearance", 1)
  Set Font = PropBag.ReadProperty("Font", Ambient.Font)
  ForeColor = PropBag.ReadProperty("ForeColor", Ambient.ForeColor)
  AutoSelect = PropBag.ReadProperty("AutoSelect", False)
  Enabled = PropBag.ReadProperty("Enabled", True)
End Sub

Private Sub UserControl_Terminate()
  Set CommonDialogCTL = Nothing
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
  PropBag.WriteProperty "AutoValidate", AutoValidate, False
  PropBag.WriteProperty "BasePath", BasePath, ""
  PropBag.WriteProperty "SelType", SelType, fnbFile
  PropBag.WriteProperty "StripBasePath", StripBasePath, True
  PropBag.WriteProperty "Text", Text, ""
  PropBag.WriteProperty "DefaultExt", DefaultExt, ""
  PropBag.WriteProperty "Filter", Filter, ""
  PropBag.WriteProperty "DialogTitle", DialogTitle, ""
  PropBag.WriteProperty "Appearance", Appearance, 1
  PropBag.WriteProperty "Font", Font, Ambient.Font
  PropBag.WriteProperty "ForeColor", ForeColor, Ambient.ForeColor
  PropBag.WriteProperty "AutoSelect", AutoSelect, False
  PropBag.WriteProperty "Enabled", Enabled, True
End Sub

Private Sub UserControl_Resize()
  If Height < 285 Then Height = 285
  txtFilename.Height = ScaleHeight
  cmdBrowse.Height = ScaleHeight
  If Appearance = 1 Then
    If ScaleWidth > cmdBrowse.Width Then txtFilename.Width = ScaleWidth - cmdBrowse.Width
    cmdBrowse.Left = txtFilename.Width
  Else
    If ScaleWidth > cmdBrowse.Width + 60 Then txtFilename.Width = ScaleWidth - cmdBrowse.Width - 60
    cmdBrowse.Left = txtFilename.Width + 60
  End If
End Sub

Public Property Get AutoValidate() As Boolean
Attribute AutoValidate.VB_Description = "Returns/sets whether the chosen filename should be validated by the FileNameBox."
  AutoValidate = m_validate
End Property

Public Property Get StripBasePath() As Boolean
Attribute StripBasePath.VB_Description = "Returns/sets whether the text box will be relative to the base path."
  StripBasePath = m_strip
End Property

Public Property Get SelType() As fnbType
Attribute SelType.VB_Description = "Returns/sets whether the browse button selects files or directories."
  SelType = m_type
End Property

Public Property Get BasePath() As String
Attribute BasePath.VB_Description = "Returns/sets the path to start the open file dialog in.  Error '-2147220477 (80040403)' will be raised if the path does not exist."
  BasePath = m_base
End Property

Public Property Get Text() As String
Attribute Text.VB_Description = "Returns/sets the text contained in the control."
Attribute Text.VB_UserMemId = -517
  Text = txtFilename.Text
End Property

Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Returns/sets a value that determines whether an object can respond to user-generated events."
Attribute Enabled.VB_UserMemId = -514
   Enabled = UserControl.Enabled
End Property

Public Property Get DialogTitle() As String
Attribute DialogTitle.VB_Description = "Sets the string displayed in the title bar of the dialog box."
  DialogTitle = m_cdTitle
End Property

Public Property Get DefaultExt() As String
Attribute DefaultExt.VB_Description = "Returns/sets the default filename extension for the dialog box."
  DefaultExt = m_cdExt
End Property

Public Property Get Filter() As String
Attribute Filter.VB_Description = "Returns/sets the filters that are displayed in the Type list box of a dialog box."
  Filter = m_cdFilter
End Property

Public Property Get Appearance() As Integer
Attribute Appearance.VB_Description = "Returns/sets whether or not an object is painted at run time with 3-D effects."
Attribute Appearance.VB_UserMemId = -520
  Appearance = UserControl.Appearance
End Property

Public Property Get Font() As Font
Attribute Font.VB_Description = "Returns a Font object."
Attribute Font.VB_UserMemId = -512
  Set Font = UserControl.Font
End Property

Public Property Get ForeColor() As OLE_COLOR
Attribute ForeColor.VB_Description = "Returns/sets the foreground color used to display text and graphics in an object."
Attribute ForeColor.VB_UserMemId = -513
  ForeColor = UserControl.ForeColor
End Property

Public Property Get AutoSelect() As Boolean
Attribute AutoSelect.VB_Description = "Returns/sets a value that determines whether the text will be automatically selected when the control gets the focus."
  AutoSelect = m_select
End Property

Public Property Let AutoSelect(a As Boolean)
  m_select = a
  PropertyChanged "AutoSelect"
End Property

Public Property Let ForeColor(ByVal fc As OLE_COLOR)
  UserControl.ForeColor = fc
  txtFilename.ForeColor = fc
  PropertyChanged "ForeColor"
End Property

Public Property Let Font(ByVal f As Font)
  UserControl.Font = f
  txtFilename.Font = f
  cmdBrowse.Font = f
  PropertyChanged "Font"
End Property

Public Property Set Font(ByVal f As Font)
  Set UserControl.Font = f
  Set txtFilename.Font = f
  Set cmdBrowse.Font = f
  PropertyChanged "Font"
End Property

Public Property Let Appearance(ByVal a As Integer)
  If a = 0 Or a = 1 Then
    UserControl.Appearance = a
    txtFilename.Appearance = a
    cmdBrowse.Appearance = a
    If a = 0 Then txtFilename.Width = ScaleWidth - cmdBrowse.Width - 60
    UserControl.BackColor = cmdBrowse.BackColor
    PropertyChanged "Appearance"
  Else
    Err.Raise 380, COMPONENT
  End If
End Property

Public Property Let Filter(ByVal f As String)
  m_cdFilter = f
  PropertyChanged "Filter"
End Property

Public Property Let DefaultExt(ByVal de As String)
  m_cdExt = de
  PropertyChanged "DefaultExt"
End Property

Public Property Let DialogTitle(ByVal dt As String)
  m_cdTitle = dt
  PropertyChanged "DialogTitle"
End Property

Public Property Let Enabled(ByVal en As Boolean)
   UserControl.Enabled = en
   txtFilename.Enabled = en
   cmdBrowse.Enabled = en
   PropertyChanged "Enabled"
End Property

Public Property Set CommonDialogCTL(ByRef ComDlg As CommonDialog)
Attribute CommonDialogCTL.VB_Description = "Must be set by passing in a CommonDialog control before the browse button will be functional."
  Set m_comdlg = ComDlg
End Property

Public Property Let AutoValidate(ByVal Validate As Boolean)
  m_validate = Validate
  PropertyChanged "AutoValidate"
End Property

Public Property Let StripBasePath(ByVal Strip As Boolean)
  m_strip = Strip
  PropertyChanged "StripBasePath"
End Property

Public Property Let SelType(ByVal FileorDir As fnbType)
  If FileorDir = fnbFile Or FileorDir = fnbFolder Then
    m_type = FileorDir
    If FileorDir = fnbFile Then
      cmdBrowse.ToolTipText = "Browse for file"
    Else
      cmdBrowse.ToolTipText = "Browse for folder"
    End If
    If FileorDir = fnbFile Then cmdBrowse.Picture = cmdFile.Picture
    If FileorDir = fnbFolder Then cmdBrowse.Picture = cmdFolder.Picture
    PropertyChanged "SelType"
  Else
    Err.Raise 380, COMPONENT
  End If
End Property

Public Property Let BasePath(ByVal Base As String)
  If Len(Base) > 0 Then
    Dim fso As FileSystemObject
    Set fso = New FileSystemObject
    If fso.FolderExists(Base) Then
      m_base = Base
      PropertyChanged "BasePath"
    Else
      Err.Raise vbObjectError + PATHDOESNOTEXIST, COMPONENT, LoadResString(PATHDOESNOTEXIST)
    End If
    Set fso = Nothing
  End If
End Property

Public Property Let Text(ByVal FileName As String)
  txtFilename.Text = FileName
  PropertyChanged "Text"
End Property

Public Property Get PathExists() As Boolean
Attribute PathExists.VB_Description = "Returns whether or not path to the currently selected file exists."
  Dim fn As String, c As Long
  Dim fso As FileSystemObject
  fn = Text
  If Len(fn) = 0 Then
    PathExists = False
    Exit Property
  End If
  If m_strip Then
    fn = BasePath & fn
  End If
  If m_type = fnbFile Then
    c = Len(fn)
    While c > 0
      If Mid$(fn, c, 1) = "\" Then
        fn = Left$(fn, c)
        c = 0
      End If
      c = c - 1
    Wend
  End If
  Set fso = New FileSystemObject
  PathExists = fso.FolderExists(fn)
End Property

Public Property Get FileExists() As Boolean
Attribute FileExists.VB_Description = "Returns whether or not the currently selected file exists."
  Dim fn As String
  Dim fso As FileSystemObject
  fn = Text
  If Len(fn) = 0 Or m_type = fnbFolder Then
    FileExists = False
    Exit Property
  End If
  If m_strip Then
    fn = BasePath & fn & DefaultExt
  End If
  Set fso = New FileSystemObject
  FileExists = fso.FileExists(fn)
End Property

Public Property Get FileName() As String
Attribute FileName.VB_Description = "Returns the fully qualified path to the selected file or folder."
  If Len(Text) > 0 Then
    FileName = Text
    If StripBasePath Then
      FileName = BasePath & FileName
      If SelType = fnbFile Then
        FileName = FileName & DefaultExt
      End If
    End If
  Else
    FileName = ""
  End If
End Property

