VERSION 5.00
Begin VB.UserControl NumberBox 
   ClientHeight    =   285
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1860
   ScaleHeight     =   285
   ScaleWidth      =   1860
   ToolboxBitmap   =   "NumberBox.ctx":0000
   Begin VB.TextBox txtNumber 
      Height          =   285
      Left            =   0
      TabIndex        =   0
      Top             =   0
      Width           =   1455
   End
End
Attribute VB_Name = "NumberBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private Const COMPONENT As Long = 555
Private Const TOOMANYPOINTS As Long = 2048

Private m_format As String
Private m_negative As Boolean
Private m_select As Boolean

Event Change()
Attribute Change.VB_Description = "This event is raised whenever the text changes."

Public Property Let FormatString(ByVal fs As String)
Attribute FormatString.VB_Description = "Returns/sets the String used to format the number."
  m_format = fs
  PropertyChanged "FormatString"
End Property

Public Property Let Value(ByVal v As Double)
  txtNumber.Text = Format$(v, FormatString)
  PropertyChanged "Value"
End Property

Public Property Let Appearance(ByVal a As Integer)
Attribute Appearance.VB_Description = "Returns/sets whether or not an object is painted at run time with 3-D effects."
  UserControl.Appearance = a
  txtNumber.Appearance = a
  PropertyChanged "Appearance"
End Property

Public Property Let Enabled(ByVal e As Boolean)
  UserControl.Enabled = e
  txtNumber.Enabled = e
  PropertyChanged "Enabled"
End Property

Public Property Set Font(ByVal f As Font)
  Set UserControl.Font = f
  Set txtNumber.Font = f
  PropertyChanged "Font"
End Property

Public Property Let Alignment(ByVal a As AlignmentConstants)
Attribute Alignment.VB_Description = "Returns/sets the alignment of a Checkbox or OptionButton, or a control's text."
  txtNumber.Alignment = a
  PropertyChanged "Alignment"
End Property

Public Property Let AllowNegative(ByVal an As Boolean)
Attribute AllowNegative.VB_Description = "Returns/sets a value that determines whether negative numbers may be entered."
  m_negative = an
  PropertyChanged "AllowNegative"
End Property

Public Property Let AutoSelect(ByVal a As Boolean)
Attribute AutoSelect.VB_Description = "Returns/sets a value that determines whether the text will be automatically selected when the control gets the focus."
  m_select = a
  PropertyChanged "AutoSelect"
End Property

Public Property Get AutoSelect() As Boolean
  AutoSelect = m_select
End Property

Public Property Get AllowNegative() As Boolean
  AllowNegative = m_negative
End Property

Public Property Get Alignment() As AlignmentConstants
  Alignment = txtNumber.Alignment
End Property

Public Property Get Font() As Font
Attribute Font.VB_Description = "Returns a Font object."
Attribute Font.VB_UserMemId = -512
  Set Font = UserControl.Font
End Property

Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Returns/sets a value that determines whether an object can respond to user-generated events."
Attribute Enabled.VB_UserMemId = -514
  Enabled = UserControl.Enabled
End Property

Public Property Get Appearance() As Integer
  Appearance = UserControl.Appearance
End Property

Public Property Get Value() As Double
Attribute Value.VB_Description = "Returns/sets the numeric value of the object.  Raises error '-2147219456 (80040800)' if there is more than one decimal point."
Attribute Value.VB_UserMemId = -517
  Dim t As String, c As Long, place As Long
  place = 0:  Value = 0: t = txtNumber.Text
  For c = 1 To Len(t)
    If Mid$(t, c, 1) <= "9" And Mid$(t, c, 1) >= "0" Then
      If place = 0 Then
        Value = 10 * Value + Val(Mid$(t, c, 1))
      Else
        Value = Value + Val(Mid$(t, c, 1)) * 10 ^ place
        place = place - 1
      End If
    ElseIf Mid$(t, c, 1) = "." Then
      If place <> 0 Then
        Err.Raise vbObjectError + TOOMANYPOINTS, COMPONENT, LoadResString(TOOMANYPOINTS)
      Else
        place = -1
      End If
    End If
  Next c
End Property

Public Property Get FormatString() As String
  FormatString = m_format
End Property

Public Sub Refresh()
Attribute Refresh.VB_Description = "Refreshes the text to show the value properly formatted."
  txtNumber.Text = Format$(Value, FormatString)
End Sub

Private Sub txtNumber_Change()
  RaiseEvent Change
End Sub

Private Sub txtNumber_GotFocus()
  If AutoSelect Then
    With txtNumber
      .SelStart = 0
      .SelLength = Len(.Text)
    End With
  End If
End Sub

Private Sub txtNumber_KeyPress(KeyAscii As Integer)
  If KeyAscii >= &H30 And KeyAscii <= &H39 Then Exit Sub
  If KeyAscii = Asc(vbBack) Or KeyAscii = Asc(",") Then Exit Sub
  If KeyAscii = Asc(".") And InStr(txtNumber.Text, ".") = 0 Then Exit Sub
  If (KeyAscii = Asc("$") Or KeyAscii = Asc("-")) _
    And (Len(txtNumber.Text) = 0 Or _
    txtNumber.SelStart = 0 And txtNumber.SelLength > 0) Then Exit Sub
  KeyAscii = 0
End Sub

Private Sub UserControl_ExitFocus()
  Refresh
End Sub

Private Sub UserControl_InitProperties()
  FormatString = ""
  Value = 0
  Appearance = 1  '3D
  Enabled = True
  Set Font = Ambient.Font
  Alignment = vbLeftJustify
  AllowNegative = False
  AutoSelect = False
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
  FormatString = PropBag.ReadProperty("FormatString", "")
  Value = PropBag.ReadProperty("Value", 0)
  Appearance = PropBag.ReadProperty("Appearance", 1)
  Enabled = PropBag.ReadProperty("Enabled", True)
  Set Font = PropBag.ReadProperty("Font", Ambient.Font)
  Alignment = PropBag.ReadProperty("Alignment", vbLeftJustify)
  AllowNegative = PropBag.ReadProperty("AllowNegative", False)
  AutoSelect = PropBag.ReadProperty("AutoSelect", False)
End Sub

Private Sub UserControl_Resize()
  If ScaleHeight < 285 Then Height = 285
  txtNumber.Height = ScaleHeight
  txtNumber.Width = ScaleWidth
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
  PropBag.WriteProperty "FormatString", FormatString, ""
  PropBag.WriteProperty "Value", Value, 0
  PropBag.WriteProperty "Appearance", Appearance, 1
  PropBag.WriteProperty "Enabled", Enabled, True
  PropBag.WriteProperty "Font", Font, Ambient.Font
  PropBag.WriteProperty "Alignment", Alignment, vbLeftJustify
  PropBag.WriteProperty "AllowNegative", AllowNegative, False
  PropBag.WriteProperty "AutoSelect", AutoSelect, False
End Sub
